/*
 * Created by Pavel Svoboda; Modified by Vladimir Bartik
 * xsvobo58 2MIT FIT VUT Brno
 */

package KnnMethod.Classes;

/**
 * The class to store the distance between testing and training item. 
 * It also stored the name of an item and its index.
 */
public class Distance implements Comparable<Distance> {

    private int indexRow;
    private String classInRow;
    private double vzdalenost;

    public Distance(int i, double l, String c) {
        this.indexRow = i;
        this.vzdalenost = l;
        this.classInRow = c;
    }

    /**
     * Gets the class name, for which the distance is stored. 
     * @return the class name, for which the distance is stored
     */
    public String getClassInRow () {
        return this.classInRow;
    }

    /**
     * Stores the class name for faster access.
     * @param classInRow class name, for which the distance is counted.
     */
    public void setClassInRow (String classInRow) {
        this.classInRow = classInRow;
    }

    /**
     * Method, which returns the value of index for the training table.
     * @return a value of index for the training table. 
     */
    public double getIndex () {
        return this.indexRow;
    }

    /**
     * Stores the index into the training table.
     * @param newIndex new value of index in the training table. 
     */
    public void setIndex (int newIndex) {
        this.indexRow = newIndex;
    }

    /**
     * Gets the object's stored distance.
     * @return object's distance
     */
    public double getVzdalenost () {
        return this.vzdalenost;
    }

    /**
     * Stores the new value of distance.
     * @param newVzdalenost the new value of object's distance
     */
    public void setVzdalenost (double newVzdalenost) {
        this.vzdalenost = newVzdalenost;
    }

    /**
     * Compares distances of objects.
     * If the distances are equal, the objects are sorted by Class
     * @param o the object to compare
     * @return negative integer, zero or positive integer, if the object's distance is
     * lower, equal or higher than a specified object o. 
     */
    public int compareTo(Distance o) {
        int result = String.valueOf(vzdalenost).compareTo(String.valueOf(o.vzdalenost));
        
        return result == 0 ? String.valueOf(classInRow).compareTo(String.valueOf(o.classInRow)) : result;
    }
}