/*
 * Created by Pavel Svoboda; Modified by Vladimir Bartik
 * xsvobo58 2MIT FIT VUT Brno
 */

package KnnMethod.UI;

import KnnMethod.Classes.MyDefaultTableModel;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import javax.swing.JTable;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

/**
 * A form, which displays statistical informations, such as accuracy of classification and number of errors for 
 * such 'k' value.
  */
public class Statistics extends javax.swing.JFrame {
    private int cisloTestu = 1;
    
    public Statistics() {        
        initComponents();
        jTableStatistic.getModel().addTableModelListener(new ListenerTableChanged());
    }

    /**
     * Returns the table model for further editation.
     * @return table model (JTable)
     */
    public JTable getTabulka () {
        return jTableStatistic;
    }

    /**
     * Gets the number of test. 
     * @return actual number of test
     */
    public int getCisloTestu () {
        return cisloTestu;
    }

    /**
     * Function, which clears all the statistics and sets the number of test to an initial value. 
     */
    private void clearStatistic() {
        ((MyDefaultTableModel)jTableStatistic.getModel()).getDataVector().clear();
        ((MyDefaultTableModel)jTableStatistic.getModel()).fireTableDataChanged();
        cisloTestu = 1;
        jLabelNumberMax.setText("%");
        jLabelNumberMin.setText("%");
        jLabelNumberAverage.setText("%");
    }

    private class ListenerTableChanged implements TableModelListener {
        public void tableChanged(TableModelEvent e) {
            cisloTestu++;
            recountFunction ();
        }        
    }

    /**
     * Function, which recounts the values of minimum and maximum. 
     */
    private void recountFunction () {
        //pokud tam je alespon jeden prvek
        if (jTableStatistic.getRowCount() > 0) {
            float maximum = (Float) jTableStatistic.getValueAt(0, 4); //4 column je uspesnost
            float minimum = maximum;
            float suma = maximum;
            float value;         
            // TODO add your handling code here:
            for(int i = 1; i < jTableStatistic.getRowCount(); i++) {
                value = (Float) jTableStatistic.getValueAt(i, 4);
                if (value > maximum) maximum = value;
                if (value < minimum) minimum = value;
                suma = suma + value;
            }
            jLabelNumberAverage.setText(suma/jTableStatistic.getRowCount() + "%");
            jLabelNumberMax.setText(maximum + "%");
            jLabelNumberMin.setText(minimum + "%");
        }        
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */    
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jLabelMax = new javax.swing.JLabel();
        jLabelNumberMax = new javax.swing.JLabel();
        jButtonClose = new javax.swing.JButton();
        jButtonClearStatistic = new javax.swing.JButton();
        jLabelMin = new javax.swing.JLabel();
        jLabelNumberMin = new javax.swing.JLabel();
        jLabelAvarege = new javax.swing.JLabel();
        jLabelNumberAverage = new javax.swing.JLabel();
        jScrollPane2 = new javax.swing.JScrollPane();
        jTableStatistic = new javax.swing.JTable();
        jButtonExportStatistic = new javax.swing.JButton();

        setTitle("K-NN method - Statistics");
        setBackground(new java.awt.Color(27, 29, 41));
        setMinimumSize(new java.awt.Dimension(550, 380));

        jPanel1.setBackground(new java.awt.Color(27, 29, 41));
        jPanel1.setMinimumSize(new java.awt.Dimension(300, 300));

        jLabelMax.setForeground(new java.awt.Color(255, 255, 255));
        jLabelMax.setText("Maximal accuracy:");

        jLabelNumberMax.setForeground(new java.awt.Color(255, 255, 255));
        jLabelNumberMax.setText("%");

        jButtonClose.setText("Close");
        jButtonClose.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonCloseActionPerformed(evt);
            }
        });

        jButtonClearStatistic.setText("Clear statistic");
        jButtonClearStatistic.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonClearStatisticActionPerformed(evt);
            }
        });

        jLabelMin.setForeground(new java.awt.Color(255, 255, 255));
        jLabelMin.setText("Minimal accuracy:");

        jLabelNumberMin.setForeground(new java.awt.Color(255, 255, 255));
        jLabelNumberMin.setText("%");

        jLabelAvarege.setForeground(new java.awt.Color(255, 255, 255));
        jLabelAvarege.setText("Average accuracy:");

        jLabelNumberAverage.setForeground(new java.awt.Color(255, 255, 255));
        jLabelNumberAverage.setText("%");

        jTableStatistic.setAutoCreateRowSorter(true);
        jTableStatistic.setModel(new MyDefaultTableModel(
            new Object [][] {

            },
            new String [] {
                "Number of test", "K value", "Number of rows", "Error count", "Accuracy"
            }
        ));
        jTableStatistic.setAutoscrolls(false);
        jTableStatistic.setEnabled(false);
        jTableStatistic.setShowHorizontalLines(false);
        jScrollPane2.setViewportView(jTableStatistic);

        jButtonExportStatistic.setText("Export");
        jButtonExportStatistic.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonExportStatisticActionPerformed(evt);
            }
        });

        javax.swing.GroupLayout jPanel1Layout = new javax.swing.GroupLayout(jPanel1);
        jPanel1.setLayout(jPanel1Layout);
        jPanel1Layout.setHorizontalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addContainerGap()
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(jLabelMin)
                            .addComponent(jLabelMax))
                        .addGap(25, 25, 25)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                            .addComponent(jLabelNumberMin, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jLabelNumberAverage, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                            .addComponent(jLabelNumberMax, javax.swing.GroupLayout.PREFERRED_SIZE, 45, javax.swing.GroupLayout.PREFERRED_SIZE)))
                    .addComponent(jLabelAvarege)
                    .addComponent(jScrollPane2))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING, false)
                    .addComponent(jButtonExportStatistic, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jButtonClearStatistic, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
                    .addComponent(jButtonClose, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 109, Short.MAX_VALUE))
                .addContainerGap())
        );
        jPanel1Layout.setVerticalGroup(
            jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(jPanel1Layout.createSequentialGroup()
                .addGap(15, 15, 15)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(jPanel1Layout.createSequentialGroup()
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelNumberMax)
                            .addComponent(jLabelMax))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelNumberMin)
                            .addComponent(jLabelMin))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                            .addComponent(jLabelAvarege)
                            .addComponent(jLabelNumberAverage)))
                    .addComponent(jButtonClose, javax.swing.GroupLayout.PREFERRED_SIZE, 43, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(jPanel1Layout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                    .addGroup(javax.swing.GroupLayout.Alignment.LEADING, jPanel1Layout.createSequentialGroup()
                        .addComponent(jButtonClearStatistic)
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addComponent(jButtonExportStatistic))
                    .addComponent(jScrollPane2, javax.swing.GroupLayout.Alignment.LEADING, javax.swing.GroupLayout.DEFAULT_SIZE, 312, Short.MAX_VALUE))
                .addContainerGap())
        );

        javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
        getContentPane().setLayout(layout);
        layout.setHorizontalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );
        layout.setVerticalGroup(
            layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addComponent(jPanel1, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE)
        );

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private void jButtonCloseActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonCloseActionPerformed
        this.setVisible(false);
    }//GEN-LAST:event_jButtonCloseActionPerformed

    private void jButtonClearStatisticActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonClearStatisticActionPerformed
        clearStatistic();
    }//GEN-LAST:event_jButtonClearStatisticActionPerformed

    private void jButtonExportStatisticActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonExportStatisticActionPerformed
        exportFunction(jTableStatistic, "");
    }//GEN-LAST:event_jButtonExportStatisticActionPerformed


    //vraci soubor, ktery byl ulozen a ktery NEEXISTOVAL!!
    //osetri pokud nazev jiz existoval , vstup nazev = "" muze byt prazdny
    //vraci null pĹ™i chybÄ›!!
    private File exportFunction (JTable tabData, String nazev) {
        File fExport = null;
        Options option = new Options();

        if(nazev.isEmpty()) {
            nazev = "export";
        }

        if (tabData.getRowCount() == 0 || tabData == null) {
            return null;
        }

        try {
            //ziska cestu k programu..
            String path = new java.io.File(".").getCanonicalPath();
            String sNumber = "";
            boolean exists = true;
            int cislo = 1;

            fExport = new File(path + "\\" + nazev + sNumber + ".csv");
            exists = fExport.exists();
            while (exists){
                sNumber = Integer.toString(cislo);
                fExport = new File(path + "\\" + nazev + sNumber + ".csv");
                exists = fExport.exists();
                cislo++;
            }

            exportTable(tabData, fExport);


        } catch (IOException ex) {
            //Logger.getLogger(knnForm.class.getName()).log(Level.SEVERE, null, ex);
        }

        return fExport;
    }

    private void exportTable(JTable table, File file) throws IOException {
        TableModel model = table.getModel();
        FileWriter out = new FileWriter(file);
        Options option = new Options();
        String separator = option.getSeparator();

        for(int i=0; i < model.getColumnCount(); i++) {
            out.write(model.getColumnName(i) + separator);
        }
        out.write("\n");
        for(int i=0; i< model.getRowCount(); i++) {
            for(int j=0; j < model.getColumnCount(); j++) {
                out.write(model.getValueAt(i,j).toString()+ separator);
            }
            out.write("\n");
        }
        out.close();        
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonClearStatistic;
    private javax.swing.JButton jButtonClose;
    private javax.swing.JButton jButtonExportStatistic;
    private javax.swing.JLabel jLabelAvarege;
    private javax.swing.JLabel jLabelMax;
    private javax.swing.JLabel jLabelMin;
    private javax.swing.JLabel jLabelNumberAverage;
    private javax.swing.JLabel jLabelNumberMax;
    private javax.swing.JLabel jLabelNumberMin;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JTable jTableStatistic;
    // End of variables declaration//GEN-END:variables

}
